@@ -2,7 +2,7 @@
  * rofi
  *
  * MIT/X11 License
- * Copyright © 2013-2017 Qball Cow <qball@gmpclient.org>
+ * Copyright © 2013-2020 Qball Cow <qball@gmpclient.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the
@@ -53,6 +53,11 @@
 
 #include "dialogs/dmenuscriptshared.h"
 
+static int dmenu_mode_init ( Mode *sw );
+static int dmenu_token_match ( const Mode *sw, rofi_int_matcher **tokens, unsigned int index );
+static cairo_surface_t *dmenu_get_icon ( const Mode *sw, unsigned int selected_line, int height );
+static char *dmenu_get_message ( const Mode *sw );
+
 static inline unsigned int bitget ( uint32_t *array, unsigned int index )
 {
     uint32_t bit = index % 32;
@@ -84,7 +89,7 @@ typedef struct
     unsigned int           num_selected_list;
     unsigned int           do_markup;
     // List with entries.
-    DmenuScriptEntry         *cmd_list;
+    DmenuScriptEntry       *cmd_list;
     unsigned int           cmd_list_real_length;
     unsigned int           cmd_list_length;
     unsigned int           only_selected;
@@ -116,14 +121,19 @@ static void read_add ( DmenuModePrivateData * pd, char *data, gsize len )
     // Init.
     pd->cmd_list[pd->cmd_list_length].icon_fetch_uid = 0;
     pd->cmd_list[pd->cmd_list_length].icon_name      = NULL;
-    char *end = strchr(data, '\0');
-    if ( end != NULL ) {
-        data_len = end-data;
-        dmenuscript_parse_entry_extras ( NULL, &(pd->cmd_list[pd->cmd_list_length]), end+1, len-data_len);
+    pd->cmd_list[pd->cmd_list_length].meta           = NULL;
+    pd->cmd_list[pd->cmd_list_length].info           = NULL;
+    char *end = data;
+    while ( end < data + len && *end != '\0' ) {
+        end++;
+    }
+    if ( end != data + len ) {
+        data_len = end - data;
+        dmenuscript_parse_entry_extras ( NULL, &( pd->cmd_list[pd->cmd_list_length] ), end + 1, len - data_len );
     }
     char *utfstr = rofi_force_utf8 ( data, data_len );
-    pd->cmd_list[pd->cmd_list_length].entry      = utfstr;
-    pd->cmd_list[pd->cmd_list_length + 1].entry  = NULL;
+    pd->cmd_list[pd->cmd_list_length].entry     = utfstr;
+    pd->cmd_list[pd->cmd_list_length + 1].entry = NULL;
 
     pd->cmd_list_length++;
 }
@@ -241,18 +251,34 @@ static gchar * dmenu_format_output_string ( const DmenuModePrivateData *pd, cons
     return retv ? retv : g_strdup ( "" );
 }
 
+static inline unsigned int get_index ( unsigned int length, int index )
+{
+    if ( index >= 0 ) {
+        return index;
+    }
+    if ( ( (unsigned int) -index ) <= length ) {
+        return length + index;
+    }
+    // Out of range.
+    return UINT_MAX;
+}
+
 static char *get_display_data ( const Mode *data, unsigned int index, int *state, G_GNUC_UNUSED GList **list, int get_entry )
 {
-    Mode                 *sw    = (Mode *) data;
-    DmenuModePrivateData *pd    = (DmenuModePrivateData *) mode_get_private_data ( sw );
-    DmenuScriptEntry    *retv = (DmenuScriptEntry *) pd->cmd_list;
+    Mode                 *sw   = (Mode *) data;
+    DmenuModePrivateData *pd   = (DmenuModePrivateData *) mode_get_private_data ( sw );
+    DmenuScriptEntry     *retv = (DmenuScriptEntry *) pd->cmd_list;
     for ( unsigned int i = 0; i < pd->num_active_list; i++ ) {
-        if ( index >= pd->active_list[i].start && index <= pd->active_list[i].stop ) {
+        unsigned int start = get_index ( pd->cmd_list_length, pd->active_list[i].start );
+        unsigned int stop  = get_index ( pd->cmd_list_length, pd->active_list[i].stop );
+        if ( index >= start && index <= stop ) {
             *state |= ACTIVE;
         }
     }
     for ( unsigned int i = 0; i < pd->num_urgent_list; i++ ) {
-        if ( index >= pd->urgent_list[i].start && index <= pd->urgent_list[i].stop ) {
+        unsigned int start = get_index ( pd->cmd_list_length, pd->urgent_list[i].start );
+        unsigned int stop  = get_index ( pd->cmd_list_length, pd->urgent_list[i].stop );
+        if ( index >= start && index <= stop ) {
             *state |= URGENT;
         }
     }
@@ -291,6 +317,8 @@ static void dmenu_mode_free ( Mode *sw )
             if ( pd->cmd_list[i].entry ) {
                 g_free ( pd->cmd_list[i].entry );
                 g_free ( pd->cmd_list[i].icon_name );
+                g_free ( pd->cmd_list[i].meta );
+                g_free ( pd->cmd_list[i].info );
             }
         }
         g_free ( pd->cmd_list );
@@ -303,6 +331,27 @@ static void dmenu_mode_free ( Mode *sw )
     }
 }
 
+#include "mode-private.h"
+/** dmenu Mode object. */
+Mode dmenu_mode =
+{
+    .name               = "dmenu",
+    .cfg_name_key       = "display-combi",
+    ._init              = dmenu_mode_init,
+    ._get_num_entries   = dmenu_mode_get_num_entries,
+    ._result            = NULL,
+    ._destroy           = dmenu_mode_free,
+    ._token_match       = dmenu_token_match,
+    ._get_display_value = get_display_data,
+    ._get_icon          = dmenu_get_icon,
+    ._get_completion    = NULL,
+    ._preprocess_input  = NULL,
+    ._get_message       = dmenu_get_message,
+    .private_data       = NULL,
+    .free               = NULL,
+    .display_name       = "dmenu"
+};
+
 static int dmenu_mode_init ( Mode *sw )
 {
     if ( mode_get_private_data ( sw ) != NULL ) {
@@ -341,6 +390,12 @@ static int dmenu_mode_init ( Mode *sw )
     // DMENU COMPATIBILITY
     find_arg_uint (  "-l", &( config.menu_lines ) );
 
+    str = NULL;
+    find_arg_str ( "-window-title", &str );
+    if ( str ) {
+        dmenu_mode.display_name = str;
+    }
+
     /**
      * Dmenu compatibility.
      * `-b` put on bottom.
@@ -386,7 +441,37 @@ static int dmenu_mode_init ( Mode *sw )
 static int dmenu_token_match ( const Mode *sw, rofi_int_matcher **tokens, unsigned int index )
 {
     DmenuModePrivateData *rmpd = (DmenuModePrivateData *) mode_get_private_data ( sw );
-    return helper_token_match ( tokens, rmpd->cmd_list[index].entry );
+    /** Strip out the markup when matching. */
+    char                 *esc = NULL;
+    if ( rmpd->do_markup ) {
+        pango_parse_markup ( rmpd->cmd_list[index].entry, -1, 0, NULL, &esc, NULL, NULL );
+    }
+    else {
+        esc = rmpd->cmd_list[index].entry;
+    }
+    if ( esc ) {
+        //        int retv = helper_token_match ( tokens, esc );
+        int match = 1;
+        if ( tokens ) {
+            for ( int j = 0; match && tokens != NULL && tokens[j] != NULL; j++ ) {
+                rofi_int_matcher *ftokens[2] = { tokens[j], NULL };
+                int              test        = 0;
+                test = helper_token_match ( ftokens, esc );
+                if ( test == tokens[j]->invert && rmpd->cmd_list[index].meta ) {
+                    test = helper_token_match ( ftokens, rmpd->cmd_list[index].meta );
+                }
+
+                if ( test == 0 ) {
+                    match = 0;
+                }
+            }
+        }
+        if ( rmpd->do_markup ) {
+            g_free ( esc );
+        }
+        return match;
+    }
+    return FALSE;
 }
 static char *dmenu_get_message ( const Mode *sw )
 {
@@ -400,7 +485,7 @@ static cairo_surface_t *dmenu_get_icon ( const Mode *sw, unsigned int selected_l
 {
     DmenuModePrivateData *pd = (DmenuModePrivateData *) mode_get_private_data ( sw );
     g_return_val_if_fail ( pd->cmd_list != NULL, NULL );
-    DmenuScriptEntry *dr = &( pd->cmd_list[selected_line] );
+    DmenuScriptEntry     *dr = &( pd->cmd_list[selected_line] );
     if ( dr->icon_name == NULL ) {
         return NULL;
     }
@@ -411,27 +496,6 @@ static cairo_surface_t *dmenu_get_icon ( const Mode *sw, unsigned int selected_l
     return rofi_icon_fetcher_get ( dr->icon_fetch_uid );
 }
 
-#include "mode-private.h"
-/** dmenu Mode object. */
-Mode dmenu_mode =
-{
-    .name               = "dmenu",
-    .cfg_name_key       = "display-combi",
-    ._init              = dmenu_mode_init,
-    ._get_num_entries   = dmenu_mode_get_num_entries,
-    ._result            = NULL,
-    ._destroy           = dmenu_mode_free,
-    ._token_match       = dmenu_token_match,
-    ._get_display_value = get_display_data,
-    ._get_icon          = dmenu_get_icon,
-    ._get_completion    = NULL,
-    ._preprocess_input  = NULL,
-    ._get_message       = dmenu_get_message,
-    .private_data       = NULL,
-    .free               = NULL,
-    .display_name       = "dmenu"
-};
-
 static void dmenu_finish ( RofiViewState *state, int retv )
 {
     if ( retv == FALSE ) {
@@ -451,19 +515,19 @@ static void dmenu_finish ( RofiViewState *state, int retv )
 static void dmenu_print_results ( DmenuModePrivateData *pd, const char *input )
 {
     DmenuScriptEntry *cmd_list = pd->cmd_list;
-    int  seen       = FALSE;
+    int              seen      = FALSE;
     if ( pd->selected_list != NULL ) {
         for ( unsigned int st = 0; st < pd->cmd_list_length; st++ ) {
             if ( bitget ( pd->selected_list, st ) ) {
                 seen = TRUE;
-                rofi_output_formatted_line ( pd->format, cmd_list[st].entry , st, input );
+                rofi_output_formatted_line ( pd->format, cmd_list[st].entry, st, input );
             }
         }
     }
     if ( !seen ) {
         const char *cmd = input;
         if ( pd->selected_line != UINT32_MAX ) {
-            cmd = cmd_list[pd->selected_line].entry ;
+            cmd = cmd_list[pd->selected_line].entry;
         }
         rofi_output_formatted_line ( pd->format, cmd, pd->selected_line, input );
     }
@@ -474,7 +538,7 @@ static void dmenu_finalize ( RofiViewState *state )
     int                  retv            = FALSE;
     DmenuModePrivateData *pd             = (DmenuModePrivateData *) rofi_view_get_mode ( state )->private_data;
     unsigned int         cmd_list_length = pd->cmd_list_length;
-    DmenuScriptEntry       *cmd_list      = pd->cmd_list;
+    DmenuScriptEntry     *cmd_list       = pd->cmd_list;
 
     char                 *input = g_strdup ( rofi_view_get_user_input ( state ) );
     pd->selected_line = rofi_view_get_selected_line ( state );;
@@ -512,6 +576,10 @@ static void dmenu_finalize ( RofiViewState *state )
                 }
             }
             else if ( ( mretv & ( MENU_OK | MENU_QUICK_SWITCH ) ) && cmd_list[pd->selected_line].entry != NULL ) {
+                if ( cmd_list[pd->selected_line].nonselectable == TRUE ) {
+                    g_free ( input );
+                    return;
+                }
                 dmenu_print_results ( pd, input );
                 retv = TRUE;
                 if ( ( mretv & MENU_QUICK_SWITCH ) ) {
@@ -537,6 +605,11 @@ static void dmenu_finalize ( RofiViewState *state )
     restart = FALSE;
     // Normal mode
     if ( ( mretv & MENU_OK  ) && pd->selected_line != UINT32_MAX && cmd_list[pd->selected_line].entry != NULL ) {
+        // Check if entry is non-selectable.
+        if ( cmd_list[pd->selected_line].nonselectable == TRUE ) {
+            g_free ( input );
+            return;
+        }
         if ( ( mretv & MENU_CUSTOM_ACTION ) && pd->multi_select ) {
             restart = TRUE;
             if ( pd->selected_list == NULL ) {
@@ -596,6 +669,7 @@ int dmenu_switcher_dialog ( void )
          find_arg ( "-selected-row" ) >= 0 ) {
         async = FALSE;
     }
+
     // Check if the subsystem is setup for reading, otherwise do not read.
     if ( pd->cancel != NULL ) {
         if ( async ) {
@@ -607,9 +681,9 @@ int dmenu_switcher_dialog ( void )
             get_dmenu_sync ( pd );
         }
     }
-    char         *input          = NULL;
-    unsigned int cmd_list_length = pd->cmd_list_length;
-    DmenuScriptEntry *cmd_list     = pd->cmd_list;
+    char             *input          = NULL;
+    unsigned int     cmd_list_length = pd->cmd_list_length;
+    DmenuScriptEntry *cmd_list       = pd->cmd_list;
 
     pd->only_selected = FALSE;
     pd->multi_select  = FALSE;
@@ -627,7 +701,7 @@ int dmenu_switcher_dialog ( void )
         }
     }
     if ( config.auto_select && cmd_list_length == 1 ) {
-        rofi_output_formatted_line ( pd->format, cmd_list[0].entry , 0, config.filter );
+        rofi_output_formatted_line ( pd->format, cmd_list[0].entry, 0, config.filter );
         return TRUE;
     }
     if ( find_arg ( "-password" ) >= 0 ) {
@@ -654,7 +728,7 @@ int dmenu_switcher_dialog ( void )
         unsigned int     i        = 0;
         for ( i = 0; i < cmd_list_length; i++ ) {
             if ( tokens == NULL || helper_token_match ( tokens, cmd_list[i].entry ) ) {
-                rofi_output_formatted_line ( pd->format, cmd_list[i].entry , i, config.filter );
+                rofi_output_formatted_line ( pd->format, cmd_list[i].entry, i, config.filter );
             }
         }
         helper_tokenize_free ( tokens );
@@ -664,6 +738,10 @@ int dmenu_switcher_dialog ( void )
     }
     find_arg_str (  "-p", &( dmenu_mode.display_name ) );
     RofiViewState *state = rofi_view_create ( &dmenu_mode, input, menu_flags, dmenu_finalize );
+
+    if ( find_arg ( "-keep-right" ) >= 0 ) {
+        rofi_view_ellipsize_start ( state );
+    }
     // @TODO we should do this better.
     if ( async && ( pd->cancel != NULL ) ) {
         rofi_view_set_overlay ( state, "Loading.. " );
@@ -684,9 +762,10 @@ void print_dmenu_options ( void )
     print_help_msg ( "-u", "[list]", "List of row indexes to mark urgent", NULL, is_term );
     print_help_msg ( "-a", "[list]", "List of row indexes to mark active", NULL, is_term );
     print_help_msg ( "-l", "[integer] ", "Number of rows to display", NULL, is_term );
+    print_help_msg ( "-window-title", "[string] ", "Set the dmenu window title", NULL, is_term );
     print_help_msg ( "-i", "", "Set filter to be case insensitive", NULL, is_term );
-    print_help_msg ( "-only-match", "", "Force selection or custom entry", NULL, is_term );
-    print_help_msg ( "-no-custom", "", "Don't accept custom entry", NULL, is_term );
+    print_help_msg ( "-only-match", "", "Force selection to be given entry, disallow no match", NULL, is_term );
+    print_help_msg ( "-no-custom", "", "Don't accept custom entry, allow no match", NULL, is_term );
     print_help_msg ( "-select", "[string]", "Select the first row that matches", NULL, is_term );
     print_help_msg ( "-password", "", "Do not show what the user inputs. Show '*' instead.", NULL, is_term );
     print_help_msg ( "-markup-rows", "", "Allow and render pango markup as input data.", NULL, is_term );
@@ -695,4 +774,5 @@ void print_dmenu_options ( void )
     print_help_msg ( "-sync", "", "Force dmenu to first read all input data, then show dialog.", NULL, is_term );
     print_help_msg ( "-async-pre-read", "[number]", "Read several entries blocking before switching to async mode", "25", is_term );
     print_help_msg ( "-w", "windowid", "Position over window with X11 windowid.", NULL, is_term );
+    print_help_msg ( "-keep-right", "", "Set ellipsize to end.", NULL, is_term );
 }
