@@ -3,7 +3,7 @@
  *
  * MIT/X11 License
  * Copyright © 2012 Sean Pringle <sean.pringle@gmail.com>
- * Copyright © 2013-2017 Qball Cow <qball@gmpclient.org>
+ * Copyright © 2013-2020 Qball Cow <qball@gmpclient.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the
@@ -79,7 +79,6 @@ void cmd_set_arguments ( int argc, char **argv )
     stored_argv = argv;
 }
 
-
 int helper_parse_setup ( char * string, char ***output, int *length, ... )
 {
     GError     *error = NULL;
@@ -161,7 +160,7 @@ static gchar *fuzzy_to_regex ( const char * input )
             g_string_append ( str, "(" );
         }
         else {
-            g_string_append ( str, ".*(" );
+            g_string_append ( str, ".*?(" );
         }
         if ( *iter == '\\' ) {
             g_string_append_c ( str, '\\' );
@@ -432,6 +431,13 @@ PangoAttrList *helper_token_match_get_pango_attr ( RofiHighlightColorStyle th, r
                         pa->start_index = start;
                         pa->end_index   = end;
                         pango_attr_list_insert ( retv, pa );
+
+                        if ( th.color.alpha < 1.0 ) {
+                            pa              = pango_attr_foreground_alpha_new ( th.color.alpha * 65535 );
+                            pa->start_index = start;
+                            pa->end_index   = end;
+                            pango_attr_list_insert ( retv, pa );
+                        }
                     }
                 }
                 g_match_info_next ( gmi, NULL );
@@ -998,6 +1004,10 @@ gboolean helper_execute_command ( const char *wd, const char *cmd, gboolean run_
         helper_parse_setup ( config.run_command, &args, &argc, "{cmd}", cmd, (char *) 0 );
     }
 
+    if ( args == NULL ) {
+        return FALSE;
+    }
+
     if ( context != NULL ) {
         if ( context->name == NULL ) {
             context->name = args[0];
@@ -1035,8 +1045,18 @@ char *helper_get_theme_path ( const char *file )
     else {
         filename = g_strconcat ( file, ".rasi", NULL );
     }
-    // Check config directory.
+    // Check config's themes directory.
     const char *cpath = g_get_user_config_dir ();
+    if ( cpath ) {
+        char *themep = g_build_filename ( cpath, "rofi", "themes", filename, NULL );
+        g_debug ( "Opening theme, testing: %s\n", themep );
+        if ( themep && g_file_test ( themep, G_FILE_TEST_EXISTS ) ) {
+            g_free ( filename );
+            return themep;
+        }
+        g_free ( themep );
+    }
+    // Check config directory.
     if ( cpath ) {
         char *themep = g_build_filename ( cpath, "rofi", filename, NULL );
         g_debug ( "Opening theme, testing: %s\n", themep );
@@ -1117,20 +1137,30 @@ cairo_surface_t* cairo_image_surface_create_from_svg ( const gchar* file, int he
 
 static void parse_pair ( char  *input, rofi_range_pair  *item )
 {
-    int                index = 0;
-    const char * const sep   = "-";
-    for ( char *token = strsep ( &input, sep ); token != NULL; token = strsep ( &input, sep ) ) {
+    // Skip leading blanks.
+    while ( input != NULL && isblank ( *input ) ) {
+        ++input;
+    }
+
+    const char *sep[]   = { "-", ":" };
+    int        pythonic = ( strchr ( input, ':' ) || input[0] == '-' ) ? 1 : 0;
+    int        index    = 0;
+
+    for (  char *token = strsep ( &input, sep[pythonic] ); token != NULL; token = strsep ( &input, sep[pythonic] ) ) {
         if ( index == 0 ) {
-            item->start = item->stop = (unsigned int) strtoul ( token, NULL, 10 );
+            item->start = item->stop = (int) strtol ( token, NULL, 10 );
             index++;
+            continue;
         }
-        else {
-            if ( token[0] == '\0' ) {
-                item->stop = 0xFFFFFFFF;
-            }
-            else{
-                item->stop = (unsigned int) strtoul ( token, NULL, 10 );
-            }
+
+        if ( token[0] == '\0' ) {
+            item->stop = -1;
+            continue;
+        }
+
+        item->stop = (int) strtol ( token, NULL, 10 );
+        if ( pythonic ) {
+            --item->stop;
         }
     }
 }
@@ -1182,12 +1212,13 @@ void rofi_output_formatted_line ( const char *format, const char *string, int se
         }
         else if ( format[i] == 'p' ) {
             char *esc = NULL;
-            pango_parse_markup(string, -1, 0, NULL, &esc, NULL, NULL);
-            if ( esc ){
+            pango_parse_markup ( string, -1, 0, NULL, &esc, NULL, NULL );
+            if ( esc ) {
                 fputs ( esc, stdout );
                 g_free ( esc );
-            } else {
-                fputs ( "invalid string" , stdout );
+            }
+            else {
+                fputs ( "invalid string", stdout );
             }
         }
         else if ( format[i] == 'q' ) {
@@ -1215,15 +1246,14 @@ void rofi_output_formatted_line ( const char *format, const char *string, int se
     fflush ( stdout );
 }
 
-
 static gboolean helper_eval_cb2 ( const GMatchInfo *info, GString *res, gpointer data )
 {
     gchar *match;
     // Get the match
-    int num_match = g_match_info_get_match_count(info);
+    int   num_match = g_match_info_get_match_count ( info );
     // Just {text} This is inside () 5.
     if ( num_match == 5 ) {
-        match = g_match_info_fetch ( info, 4);
+        match = g_match_info_fetch ( info, 4 );
         if ( match != NULL ) {
             // Lookup the match, so we can replace it.
             gchar *r = g_hash_table_lookup ( (GHashTable *) data, match );
@@ -1237,21 +1267,21 @@ static gboolean helper_eval_cb2 ( const GMatchInfo *info, GString *res, gpointer
     }
     // {} with [] guard around it.
     else if ( num_match == 4 ) {
-        match = g_match_info_fetch ( info, 2);
+        match = g_match_info_fetch ( info, 2 );
         if ( match != NULL ) {
             // Lookup the match, so we can replace it.
             gchar *r = g_hash_table_lookup ( (GHashTable *) data, match );
             if ( r != NULL ) {
                 // Add (optional) prefix
-                gchar *prefix = g_match_info_fetch (info, 1);
+                gchar *prefix = g_match_info_fetch ( info, 1 );
                 g_string_append ( res, prefix );
-                g_free (prefix );
+                g_free ( prefix );
                 // Append the replacement to the string.
                 g_string_append ( res, r );
                 // Add (optional) postfix
-                gchar *post = g_match_info_fetch (info, 3);
+                gchar *post = g_match_info_fetch ( info, 3 );
                 g_string_append ( res, post );
-                g_free (post );
+                g_free ( post );
             }
             // Free match.
             g_free ( match );
@@ -1266,7 +1296,7 @@ char *helper_string_replace_if_exists ( char * string, ... )
 {
     GHashTable *h;
     h = g_hash_table_new ( g_str_hash, g_str_equal );
-    va_list ap;
+    va_list    ap;
     va_start ( ap, string );
     // Add list from variable arguments.
     while ( 1 ) {
@@ -1298,12 +1328,12 @@ char *helper_string_replace_if_exists ( char * string, ... )
  */
 char *helper_string_replace_if_exists_v ( char * string, GHashTable *h )
 {
-    GError  *error = NULL;
-    char    *res   = NULL;
+    GError *error = NULL;
+    char   *res   = NULL;
 
     // Replace hits within {-\w+}.
     GRegex *reg = g_regex_new ( "\\[(.*)({[-\\w]+})(.*)\\]|({[\\w-]+})", 0, 0, &error );
-    if ( error == NULL ){
+    if ( error == NULL ) {
         res = g_regex_replace_eval ( reg, string, -1, 0, 0, helper_eval_cb2, h, &error );
     }
     // Free regex.
