@@ -3,7 +3,7 @@
  *
  * MIT/X11 License
  * Copyright © 2012 Sean Pringle <sean.pringle@gmail.com>
- * Copyright © 2013-2017 Qball Cow <qball@gmpclient.org>
+ * Copyright © 2013-2020 Qball Cow <qball@gmpclient.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the
@@ -115,9 +115,11 @@ NkBindings *bindings = NULL;
 GMainLoop *main_loop = NULL;
 
 /** Flag indicating we are in dmenu mode. */
-static int dmenu_mode = FALSE;
+static int      dmenu_mode = FALSE;
 /** Rofi's return code */
-int        return_code = EXIT_SUCCESS;
+int             return_code = EXIT_SUCCESS;
+
+static gboolean old_config_format = FALSE;
 
 void process_result ( RofiViewState *state );
 
@@ -189,6 +191,13 @@ static void run_switcher ( ModeMode mode )
     }
     curr_switcher = mode;
     RofiViewState * state = rofi_view_create ( modi[mode], config.filter, 0, process_result );
+
+    // User can pre-select a row.
+    if ( find_arg ( "-selected-row" ) >= 0 ) {
+        unsigned int sr = 0;
+        find_arg_uint (  "-selected-row", &( sr ) );
+        rofi_view_set_selected_line ( state, sr );
+    }
     if ( state ) {
         rofi_view_set_active ( state );
     }
@@ -276,12 +285,13 @@ static void print_main_application_options ( int is_term )
     print_help_msg ( "-dump-xresources", "", "Dump the current configuration in Xresources format and exit.", NULL, is_term );
     print_help_msg ( "-e", "[string]", "Show a dialog displaying the passed message and exit.", NULL, is_term );
     print_help_msg ( "-markup", "", "Enable pango markup where possible.", NULL, is_term );
-    print_help_msg ( "-normal-window", "", "In dmenu mode, behave as a normal window. (experimental)", NULL, is_term );
+    print_help_msg ( "-normal-window", "", "Behave as a normal window. (experimental)", NULL, is_term );
     print_help_msg ( "-show", "[mode]", "Show the mode 'mode' and exit. The mode has to be enabled.", NULL, is_term );
     print_help_msg ( "-no-lazy-grab", "", "Disable lazy grab that, when fail to grab keyboard, does not block but retry later.", NULL, is_term );
     print_help_msg ( "-no-plugins", "", "Disable loading of external plugins.", NULL, is_term );
-    print_help_msg ( "-plugin-path", "", "Directory used to search for rofi plugins.", NULL, is_term );
+    print_help_msg ( "-plugin-path", "", "Directory used to search for rofi plugins. *DEPRECATED*", NULL, is_term );
     print_help_msg ( "-dump-config", "", "Dump the current configuration in rasi format and exit.", NULL, is_term );
+    print_help_msg ( "-upgrade-config", "", "Upgrade the old-style configuration fiel in the new rasi format and exit.", NULL, is_term );
     print_help_msg ( "-dump-theme", "", "Dump the current theme in rasi format and exit.", NULL, is_term );
 }
 static void help ( G_GNUC_UNUSED int argc, char **argv )
@@ -361,8 +371,8 @@ static void help_print_disabled_mode ( const char *mode )
 }
 static void help_print_mode_not_found ( const char *mode )
 {
-    GString *str = g_string_new ("");
-    g_string_printf(str, "Mode %s is not found.\nThe following modi are known:\n", mode );
+    GString *str = g_string_new ( "" );
+    g_string_printf ( str, "Mode %s is not found.\nThe following modi are known:\n", mode );
     for ( unsigned int i = 0; i < num_available_modi; i++ ) {
         gboolean active = FALSE;
         for ( unsigned int j = 0; j < num_modi; j++ ) {
@@ -371,10 +381,10 @@ static void help_print_mode_not_found ( const char *mode )
                 break;
             }
         }
-        g_string_append_printf (str, "        * %s%s\n",
-                active ? "+" : "",
-                available_modi[i]->name
-                );
+        g_string_append_printf ( str, "        * %s%s\n",
+                                 active ? "+" : "",
+                                 available_modi[i]->name
+                                 );
     }
     rofi_add_error_message ( str );
 }
@@ -494,6 +504,7 @@ static gboolean rofi_collect_modi_add ( Mode *mode )
 
 static void rofi_collect_modi_dir ( const char *base_dir )
 {
+    g_debug ( "Looking into: %s for plugins", base_dir );
     GDir *dir = g_dir_open ( base_dir, 0, NULL );
     if ( dir ) {
         const char *dn = NULL;
@@ -501,7 +512,8 @@ static void rofi_collect_modi_dir ( const char *base_dir )
             if ( !g_str_has_suffix ( dn, G_MODULE_SUFFIX ) ) {
                 continue;
             }
-            char    *fn  = g_build_filename ( base_dir, dn, NULL );
+            char    *fn = g_build_filename ( base_dir, dn, NULL );
+            g_debug ( "Trying to open: %s plugin", fn );
             GModule *mod = g_module_open ( fn, G_MODULE_BIND_LAZY | G_MODULE_BIND_LOCAL );
             if ( mod ) {
                 Mode *m = NULL;
@@ -552,6 +564,15 @@ static void rofi_collect_modi ( void )
         find_arg_str ( "-plugin-path", &( config.plugin_path ) );
         g_debug ( "Parse plugin path: %s", config.plugin_path );
         rofi_collect_modi_dir ( config.plugin_path );
+        /* ROFI_PLUGIN_PATH */
+        const char *path = g_getenv ( "ROFI_PLUGIN_PATH" );
+        if ( path != NULL ) {
+            gchar ** paths = g_strsplit ( path, ":", -1 );
+            for ( unsigned int i = 0; paths[i]; i++ ) {
+                rofi_collect_modi_dir ( paths[i] );
+            }
+            g_strfreev ( paths );
+        }
     }
 }
 
@@ -660,7 +681,6 @@ static void show_error_dialog ()
     rofi_view_error_dialog ( emesg->str, ERROR_MSG_MARKUP );
     g_string_free ( emesg, TRUE );
     rofi_set_return_code ( EX_DATAERR );
-
 }
 
 static gboolean startup ( G_GNUC_UNUSED gpointer data )
@@ -768,6 +788,17 @@ int main ( int argc, char *argv[] )
         return EXIT_SUCCESS;
     }
 
+    {
+        const char *ro_pid = g_getenv ( "ROFI_OUTSIDE" );
+        if ( ro_pid != NULL ) {
+            int ro_pidi = g_ascii_strtoll ( ro_pid, NULL, 0 );
+            if ( kill ( ro_pidi, 0 ) == 0 ) {
+                printf ( "Do not launch rofi from inside rofi.\r\n" );
+                return EXIT_FAILURE;
+            }
+        }
+    }
+
     // Detect if we are in dmenu mode.
     // This has two possible causes.
     // 1 the user specifies it on the command-line.
@@ -849,13 +880,55 @@ int main ( int argc, char *argv[] )
 
     if ( find_arg ( "-no-config" ) < 0 ) {
         // Load distro default settings
-        gchar *etc = g_build_filename ( SYSCONFDIR, "rofi.conf", NULL );
-        if ( g_file_test ( etc, G_FILE_TEST_IS_REGULAR ) ) {
-            config_parse_xresource_options_file ( etc );
+        gboolean           found_system = FALSE;
+        const char * const * dirs       = g_get_system_config_dirs ();
+        if ( dirs ) {
+            for ( unsigned int i = 0; !found_system && dirs[i]; i++ ) {
+                /** New format. */
+                gchar *etc = g_build_filename ( dirs[i], "rofi.rasi", NULL );
+                g_debug ( "Look for default config file: %s", etc );
+                if ( g_file_test ( etc, G_FILE_TEST_IS_REGULAR ) ) {
+                    g_debug ( "Parsing: %s", etc );
+                    rofi_theme_parse_file ( etc );
+                    found_system = TRUE;
+                }
+                else {
+                    /** Old format. */
+                    gchar *xetc = g_build_filename ( dirs[i], "rofi.conf", NULL );
+                    g_debug ( "Look for default config file: %s", xetc );
+                    if ( g_file_test ( xetc, G_FILE_TEST_IS_REGULAR ) ) {
+                        config_parse_xresource_options_file ( xetc );
+                        old_config_format = TRUE;
+                        found_system      = TRUE;
+                    }
+                    g_free ( xetc );
+                }
+                g_free ( etc );
+            }
+        }
+        if ( !found_system  ) {
+            /** New format. */
+            gchar *etc = g_build_filename ( SYSCONFDIR, "rofi.rasi", NULL );
+            g_debug ( "Look for default config file: %s", etc );
+            if ( g_file_test ( etc, G_FILE_TEST_IS_REGULAR ) ) {
+                g_debug ( "Look for default config file: %s", etc );
+                rofi_theme_parse_file ( etc );
+            }
+            else {
+                /** Old format. */
+                gchar *xetc = g_build_filename ( SYSCONFDIR, "rofi.conf", NULL );
+                g_debug ( "Look for default config file: %s", xetc );
+                if ( g_file_test ( xetc, G_FILE_TEST_IS_REGULAR ) ) {
+                    config_parse_xresource_options_file ( xetc );
+                    old_config_format = TRUE;
+                }
+                g_free ( xetc );
+            }
+            g_free ( etc );
         }
-        g_free ( etc );
         // Load in config from X resources.
         config_parse_xresource_options ( xcb );
+
         if ( config_path_new && g_file_test ( config_path_new, G_FILE_TEST_IS_REGULAR ) ) {
             if ( rofi_theme_parse_file ( config_path_new ) ) {
                 rofi_theme_free ( rofi_theme );
@@ -865,7 +938,10 @@ int main ( int argc, char *argv[] )
         else {
             g_free ( config_path_new );
             config_path_new = NULL;
-            config_parse_xresource_options_file ( config_path );
+            if ( g_file_test ( config_path, G_FILE_TEST_IS_REGULAR ) ) {
+                config_parse_xresource_options_file ( config_path );
+                old_config_format = TRUE;
+            }
         }
     }
     find_arg_str ( "-theme", &( config.theme ) );
@@ -882,6 +958,10 @@ int main ( int argc, char *argv[] )
     config_parse_cmd_options ( );
     TICK_N ( "Load cmd config " );
 
+    if ( old_config_format ) {
+        g_warning ( "The old Xresources based configuration format is deprecated." );
+        g_warning ( "Please upgrade: rofi -upgrade-config." );
+    }
     parse_keys_abe ( bindings );
 
     // Get the path to the cache dir.
@@ -958,8 +1038,54 @@ int main ( int argc, char *argv[] )
         cleanup ();
         return EXIT_SUCCESS;
     }
+    if ( find_arg ( "-upgrade-config" ) >= 0 ) {
+        setup_modi ();
+
+        for ( unsigned int i = 0; i < num_modi; i++ ) {
+            mode_init ( modi[i] );
+        }
+
+        const char *cpath = g_get_user_config_dir ();
+        if ( cpath ) {
+            char *fcpath = g_build_filename ( cpath, "rofi", NULL );
+            if ( !g_file_test ( fcpath, G_FILE_TEST_IS_DIR ) && g_mkdir_with_parents ( fcpath, 0700 ) < 0 ) {
+                g_warning ( "Failed to create rofi configuration directory: %s", fcpath );
+                cleanup ();
+                g_free ( fcpath );
+                return EXIT_FAILURE;
+            }
+            g_free ( fcpath );
+            fcpath = g_build_filename ( cpath, "rofi", "config.rasi", NULL );
+            if ( g_file_test ( fcpath, G_FILE_TEST_IS_REGULAR ) ) {
+                g_warning ( "New configuration file already exists: %s", fcpath );
+                cleanup ();
+                g_free ( fcpath );
+                return EXIT_FAILURE;
+            }
+            FILE *fd = fopen ( fcpath, "w" );
+            if ( fd == NULL ) {
+                g_warning ( "Failed to open new rofi configuration file: %s: %s", fcpath, strerror ( errno ) );
+                cleanup ();
+                g_free ( fcpath );
+                return EXIT_FAILURE;
+            }
+            config_parse_dump_config_rasi_format ( fd, TRUE );
+            fprintf ( stdout, "\n***** Generated configuration file in: %s *****\n", fcpath );
+
+            fflush ( fd );
+            fclose ( fd );
+            g_free ( fcpath );
+        }
+        else {
+            g_warning ( "Failed to get user configuration directory." );
+            cleanup ();
+            return EXIT_FAILURE;
+        }
+        cleanup ();
+        return EXIT_SUCCESS;
+    }
     if ( find_arg ( "-dump-config" ) >= 0 ) {
-        config_parse_dump_config_rasi_format ( FALSE );
+        config_parse_dump_config_rasi_format ( stdout, FALSE );
         cleanup ();
         return EXIT_SUCCESS;
     }
@@ -980,17 +1106,24 @@ int main ( int argc, char *argv[] )
     if ( find_arg_uint ( "-record-screenshots", &interval ) ) {
         g_timeout_add ( 1000 / (double) interval, record, NULL );
     }
+    if ( find_arg ( "-benchmark-ui" ) >= 0 ) {
+        config.benchmark_ui = TRUE;
+    }
 
     rofi_view_workers_initialize ();
+    TICK_N ( "Workers initialize" );
     rofi_icon_fetcher_init ( );
+    TICK_N ( "Icon fetcher initialize" );
 
     // Create pid file
     int pfd = create_pid_file ( pidfile );
+    TICK_N ( "Pid file created" );
     if ( pfd < 0 ) {
         cleanup ();
         return EXIT_FAILURE;
     }
     textbox_setup ();
+    TICK_N ( "Text box setup" );
 
     if ( !display_late_setup () ) {
         g_warning ( "Failed to properly finish display setup" );
@@ -999,6 +1132,9 @@ int main ( int argc, char *argv[] )
     }
     TICK_N ( "Setup late Display" );
 
+    rofi_theme_parse_process_conditionals ();
+    TICK_N ( "Theme setup" );
+
     // Setup signal handling sources.
     // SIGINT
     g_unix_signal_add ( SIGINT, main_loop_signal_handler_int, NULL );
