@@ -2,7 +2,7 @@
  * rofi
  *
  * MIT/X11 License
- * Copyright © 2013-2017 Qball Cow <qball@gmpclient.org>
+ * Copyright © 2013-2020 Qball Cow <qball@gmpclient.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the
@@ -47,7 +47,8 @@
 void yyerror ( YYLTYPE *yylloc, const char *, const char * );
 static gboolean distance_compare ( RofiDistance d, RofiDistance e )
 {
-    return d.type == e.type && d.distance == e.distance && d.style == e.style;
+    // TODO UPDATE
+    return d.base.type == e.base.type && d.base.distance == e.base.distance && d.style == e.style;
 }
 
 static gpointer rofi_g_list_strdup ( gconstpointer data, G_GNUC_UNUSED gpointer user_data )
@@ -80,6 +81,31 @@ Property *rofi_theme_property_create ( PropertyType type )
     retv->type = type;
     return retv;
 }
+
+static RofiDistanceUnit  *rofi_theme_property_copy_distance_unit ( RofiDistanceUnit *unit )
+{
+    RofiDistanceUnit *retv = g_slice_new0 ( RofiDistanceUnit );
+    *retv = *unit;
+    if ( unit->left ) {
+        retv->left = rofi_theme_property_copy_distance_unit ( unit->left );
+    }
+    if ( unit->right ) {
+        retv->right = rofi_theme_property_copy_distance_unit ( unit->right );
+    }
+    return retv;
+}
+RofiDistance rofi_theme_property_copy_distance  ( RofiDistance const distance )
+{
+    RofiDistance retv = distance;
+    if ( distance.base.left ) {
+        retv.base.left = rofi_theme_property_copy_distance_unit ( distance.base.left );
+    }
+    if ( distance.base.right ) {
+        retv.base.right = rofi_theme_property_copy_distance_unit ( distance.base.right );
+    }
+    return retv;
+}
+
 Property* rofi_theme_property_copy ( Property *p )
 {
     Property *retv = rofi_theme_property_create ( p->type );
@@ -96,16 +122,49 @@ Property* rofi_theme_property_copy ( Property *p )
     case P_LINK:
         retv->value.link.name = g_strdup ( p->value.link.name );
         retv->value.link.ref  = NULL;
-        if ( p->value.link.def_value ){
-            retv->value.link.def_value = rofi_theme_property_copy(p->value.link.def_value); 
+        if ( p->value.link.def_value ) {
+            retv->value.link.def_value = rofi_theme_property_copy ( p->value.link.def_value );
         }
         break;
+    case P_PADDING:
+    {
+        retv->value                = p->value;
+        retv->value.padding.top    = rofi_theme_property_copy_distance ( p->value.padding.top );
+        retv->value.padding.left   = rofi_theme_property_copy_distance ( p->value.padding.left );
+        retv->value.padding.bottom = rofi_theme_property_copy_distance ( p->value.padding.bottom );
+        retv->value.padding.right  = rofi_theme_property_copy_distance ( p->value.padding.right );
+        break;
+    }
     default:
         retv->value = p->value;
     }
     return retv;
 }
 
+static void rofi_theme_distance_unit_property_free ( RofiDistanceUnit *unit )
+{
+    if ( unit->left ) {
+        rofi_theme_distance_unit_property_free ( unit->left );
+        unit->left = NULL;
+    }
+    if ( unit->right ) {
+        rofi_theme_distance_unit_property_free ( unit->right );
+        unit->right = NULL;
+    }
+    g_slice_free ( RofiDistanceUnit, unit );
+}
+static void rofi_theme_distance_property_free ( RofiDistance *distance )
+{
+    if ( distance->base.left ) {
+        rofi_theme_distance_unit_property_free ( distance->base.left );
+        distance->base.left = NULL;
+    }
+    if ( distance->base.right ) {
+        rofi_theme_distance_unit_property_free ( distance->base.right );
+        distance->base.right = NULL;
+    }
+}
+
 void rofi_theme_property_free ( Property *p )
 {
     if ( p == NULL ) {
@@ -121,14 +180,67 @@ void rofi_theme_property_free ( Property *p )
             rofi_theme_property_free ( p->value.link.def_value );
         }
     }
+    if ( p->type == P_PADDING ) {
+        rofi_theme_distance_property_free ( &( p->value.padding.top ) );
+        rofi_theme_distance_property_free ( &( p->value.padding.right ) );
+        rofi_theme_distance_property_free ( &( p->value.padding.bottom ) );
+        rofi_theme_distance_property_free ( &( p->value.padding.left ) );
+    }
     g_slice_free ( Property, p );
 }
 
+/**
+ * This function is a hack to insert backward support for older theme with the updated listvie structure.
+ */
+static void rofi_theme_insert_listview_backwards_fix ( void )
+{
+    GHashTable  *table = g_hash_table_new_full ( g_str_hash, g_str_equal, NULL, (GDestroyNotify) rofi_theme_property_free );
+    ThemeWidget *t     = rofi_theme_find_or_create_name ( rofi_theme, "element" );
+    ThemeWidget *tt    = rofi_theme_find_or_create_name ( rofi_theme, "element-text" );
+    ThemeWidget *ti    = rofi_theme_find_or_create_name ( rofi_theme, "element-icon" );
+
+    // Inherit text color
+    Property *ptc = rofi_theme_property_create ( P_INHERIT );
+    ptc->name = g_strdup ( "text-color" );
+    g_hash_table_replace ( table, ptc->name, ptc );
+    // Transparent background
+    Property *ptb = rofi_theme_property_create ( P_COLOR );
+    ptb->name              = g_strdup ( "background-color" );
+    ptb->value.color.red   = 0.0;
+    ptb->value.color.green = 0.0;
+    ptb->value.color.blue  = 0.0;
+    ptb->value.color.alpha = 0.0;
+    g_hash_table_replace ( table, ptb->name, ptb );
+
+    rofi_theme_widget_add_properties ( tt, table );
+
+    RofiDistance dsize = (RofiDistance){ .base = { 1.2, ROFI_PU_CH, ROFI_DISTANCE_MODIFIER_NONE, NULL, NULL }, .style = ROFI_HL_SOLID };
+    Property     *pts  = rofi_theme_property_create ( P_PADDING );
+    pts->value.padding.top = pts->value.padding.right = pts->value.padding.bottom = pts->value.padding.left = dsize;
+    pts->name              = g_strdup ( "size" );
+    g_hash_table_replace ( table, pts->name, pts );
+
+    rofi_theme_widget_add_properties ( ti, table );
+
+    /** Add spacing between icon and text. */
+    g_hash_table_destroy ( table );
+    table = g_hash_table_new_full ( g_str_hash, g_str_equal, NULL, (GDestroyNotify) rofi_theme_property_free );
+    Property     *psp = rofi_theme_property_create ( P_PADDING );
+    psp->name = g_strdup ( "spacing" );
+    RofiDistance d = (RofiDistance){ .base = { 5, ROFI_PU_PX, ROFI_DISTANCE_MODIFIER_NONE, NULL, NULL }, .style = ROFI_HL_SOLID };
+    psp->value.padding = (RofiPadding){ d, d, d, d };
+    g_hash_table_replace ( table, psp->name, psp );
+    rofi_theme_widget_add_properties ( t, table );
+    g_hash_table_destroy ( table );
+}
+
 void rofi_theme_reset ( void )
 {
     rofi_theme_free ( rofi_theme );
     rofi_theme       = g_slice_new0 ( ThemeWidget );
     rofi_theme->name = g_strdup ( "Root" );
+    // Hack to fix backwards compatibility.
+    rofi_theme_insert_listview_backwards_fix ( );
 }
 
 void rofi_theme_free ( ThemeWidget *widget )
@@ -140,6 +252,9 @@ void rofi_theme_free ( ThemeWidget *widget )
         g_hash_table_destroy ( widget->properties );
         widget->properties = NULL;
     }
+    if ( widget->media ) {
+        g_slice_free ( ThemeMedia, widget->media );
+    }
     for ( unsigned int i = 0; i < widget->num_widgets; i++ ) {
         rofi_theme_free ( widget->widgets[i] );
     }
@@ -157,18 +272,69 @@ inline static void printf_double ( double d )
     g_ascii_formatd ( buf, G_ASCII_DTOSTR_BUF_SIZE, "%.4lf", d );
     fputs ( buf, stdout );
 }
+
+static void rofi_theme_print_distance_unit ( RofiDistanceUnit *unit )
+{
+    if ( unit->modtype == ROFI_DISTANCE_MODIFIER_GROUP ) {
+        fputs ( "( ", stdout );
+    }
+    if ( unit->left ) {
+        rofi_theme_print_distance_unit ( unit->left );
+    }
+
+    if ( unit->modtype == ROFI_DISTANCE_MODIFIER_ADD ) {
+        fputs ( " + ", stdout );
+    }
+    else if ( unit->modtype == ROFI_DISTANCE_MODIFIER_SUBTRACT ) {
+        fputs ( " - ", stdout );
+    }
+    else if ( unit->modtype == ROFI_DISTANCE_MODIFIER_DIVIDE ) {
+        fputs ( " / ", stdout );
+    }
+    else if ( unit->modtype == ROFI_DISTANCE_MODIFIER_MULTIPLY ) {
+        fputs ( " * ", stdout );
+    }
+    else if ( unit->modtype == ROFI_DISTANCE_MODIFIER_MODULO ) {
+        fputs ( " % ", stdout );
+    }
+    if ( unit->right ) {
+        rofi_theme_print_distance_unit ( unit->right );
+    }
+
+    if ( unit->modtype == ROFI_DISTANCE_MODIFIER_NONE ) {
+        if ( unit->type == ROFI_PU_PX ) {
+            printf ( "%upx ", (unsigned int) unit->distance );
+        }
+        else if ( unit->type == ROFI_PU_MM ) {
+            printf_double ( unit->distance );
+            fputs ( "mm ", stdout );
+        }
+        else if ( unit->type == ROFI_PU_PERCENT ) {
+            printf_double ( unit->distance );
+            fputs ( "% ", stdout );
+        }
+        else if ( unit->type == ROFI_PU_CH ) {
+            printf_double ( unit->distance );
+            fputs ( "ch ", stdout );
+        }
+        else {
+            printf_double ( unit->distance );
+            fputs ( "em ", stdout );
+        }
+    }
+    if ( unit->modtype == ROFI_DISTANCE_MODIFIER_GROUP ) {
+        fputs ( " )", stdout );
+    }
+}
+
 static void rofi_theme_print_distance ( RofiDistance d )
 {
-    if ( d.type == ROFI_PU_PX ) {
-        printf ( "%upx ", (unsigned int) d.distance );
+    if ( d.base.modtype == ROFI_DISTANCE_MODIFIER_GROUP ) {
+        fputs ( "calc( ", stdout );
     }
-    else if ( d.type == ROFI_PU_PERCENT ) {
-        printf_double ( d.distance );
-        fputs ( "%% ", stdout );
-    }
-    else {
-        printf_double ( d.distance );
-        fputs ( "em ", stdout );
+    rofi_theme_print_distance_unit ( &( d.base ) );
+    if ( d.base.modtype == ROFI_DISTANCE_MODIFIER_GROUP ) {
+        fputs ( ")", stdout );
     }
     if ( d.style == ROFI_HL_DASH ) {
         printf ( "dash " );
@@ -191,102 +357,102 @@ static void int_rofi_theme_print_property ( Property *p )
 {
     switch ( p->type )
     {
-        case P_LIST:
-            printf ( "[ " );
-            for ( GList *iter = p->value.list; iter != NULL; iter = g_list_next ( iter ) ) {
-                printf ( "%s", (char *) ( iter->data ) );
-                if ( iter->next != NULL ) {
-                    printf ( "," );
-                }
+    case P_LIST:
+        printf ( "[ " );
+        for ( GList *iter = p->value.list; iter != NULL; iter = g_list_next ( iter ) ) {
+            printf ( "%s", (char *) ( iter->data ) );
+            if ( iter->next != NULL ) {
+                printf ( "," );
             }
-            printf ( " ]" );
-            break;
-        case P_ORIENTATION:
-            printf ( "%s", ( p->value.i == ROFI_ORIENTATION_HORIZONTAL ) ? "horizontal" : "vertical" );
-            break;
-        case P_HIGHLIGHT:
-            if ( p->value.highlight.style & ROFI_HL_BOLD ) {
-                printf ( "bold " );
-            }
-            if ( p->value.highlight.style & ROFI_HL_UNDERLINE ) {
-                printf ( "underline " );
-            }
-            if ( p->value.highlight.style & ROFI_HL_STRIKETHROUGH ) {
-                printf ( "strikethrough " );
-            }
-            if ( p->value.highlight.style & ROFI_HL_ITALIC ) {
-                printf ( "italic " );
-            }
-            if ( p->value.highlight.style & ROFI_HL_COLOR ) {
-                printf ( "rgba ( %.0f, %.0f, %.0f, %.0f %% )",
-                        ( p->value.highlight.color.red * 255.0 ),
-                        ( p->value.highlight.color.green * 255.0 ),
-                        ( p->value.highlight.color.blue * 255.0 ),
-                        ( p->value.highlight.color.alpha * 100.0 ) );
-            }
-            break;
-        case P_POSITION:
-            printf ( "%s", WindowLocationStr[p->value.i] );
-            break;
-        case P_STRING:
-            printf ( "\"%s\"", p->value.s );
-            break;
-        case P_INTEGER:
-            printf ( "%d", p->value.i );
-            break;
-        case P_DOUBLE:
-            printf ( "%.2f", p->value.f );
-            break;
-        case P_BOOLEAN:
-            printf ( "%s", p->value.b ? "true" : "false" );
-            break;
-        case P_COLOR:
+        }
+        printf ( " ]" );
+        break;
+    case P_ORIENTATION:
+        printf ( "%s", ( p->value.i == ROFI_ORIENTATION_HORIZONTAL ) ? "horizontal" : "vertical" );
+        break;
+    case P_HIGHLIGHT:
+        if ( p->value.highlight.style & ROFI_HL_BOLD ) {
+            printf ( "bold " );
+        }
+        if ( p->value.highlight.style & ROFI_HL_UNDERLINE ) {
+            printf ( "underline " );
+        }
+        if ( p->value.highlight.style & ROFI_HL_STRIKETHROUGH ) {
+            printf ( "strikethrough " );
+        }
+        if ( p->value.highlight.style & ROFI_HL_ITALIC ) {
+            printf ( "italic " );
+        }
+        if ( p->value.highlight.style & ROFI_HL_COLOR ) {
             printf ( "rgba ( %.0f, %.0f, %.0f, %.0f %% )",
-                    ( p->value.color.red * 255.0 ),
-                    ( p->value.color.green * 255.0 ),
-                    ( p->value.color.blue * 255.0 ),
-                    ( p->value.color.alpha * 100.0 ) );
-            break;
-        case P_PADDING:
-            if ( distance_compare ( p->value.padding.top, p->value.padding.bottom ) &&
-                    distance_compare ( p->value.padding.left, p->value.padding.right ) &&
-                    distance_compare ( p->value.padding.left, p->value.padding.top ) ) {
-                rofi_theme_print_distance ( p->value.padding.left );
-            }
-            else if ( distance_compare ( p->value.padding.top, p->value.padding.bottom ) &&
-                    distance_compare ( p->value.padding.left, p->value.padding.right ) ) {
-                rofi_theme_print_distance ( p->value.padding.top );
-                rofi_theme_print_distance ( p->value.padding.left );
-            }
-            else if ( !distance_compare ( p->value.padding.top, p->value.padding.bottom ) &&
-                    distance_compare ( p->value.padding.left, p->value.padding.right ) ) {
-                rofi_theme_print_distance ( p->value.padding.top );
-                rofi_theme_print_distance ( p->value.padding.left );
-                rofi_theme_print_distance ( p->value.padding.bottom );
-            }
-            else {
-                rofi_theme_print_distance ( p->value.padding.top );
-                rofi_theme_print_distance ( p->value.padding.right );
-                rofi_theme_print_distance ( p->value.padding.bottom );
-                rofi_theme_print_distance ( p->value.padding.left );
-            }
-            break;
-        case P_LINK:
-            if ( p->value.link.def_value) {
-                printf( "var( %s, ", p->value.link.name );
-                int_rofi_theme_print_property ( p->value.link.def_value );
-                printf (")");
-            }else {
-                printf ( "var(%s)", p->value.link.name );
-            }
-            break;
-        case P_INHERIT:
-            printf ( "inherit" );
-            break;
-        default:
-            break;
+                     ( p->value.highlight.color.red * 255.0 ),
+                     ( p->value.highlight.color.green * 255.0 ),
+                     ( p->value.highlight.color.blue * 255.0 ),
+                     ( p->value.highlight.color.alpha * 100.0 ) );
+        }
+        break;
+    case P_POSITION:
+        printf ( "%s", WindowLocationStr[p->value.i] );
+        break;
+    case P_STRING:
+        printf ( "\"%s\"", p->value.s );
+        break;
+    case P_INTEGER:
+        printf ( "%d", p->value.i );
+        break;
+    case P_DOUBLE:
+        printf ( "%.2f", p->value.f );
+        break;
+    case P_BOOLEAN:
+        printf ( "%s", p->value.b ? "true" : "false" );
+        break;
+    case P_COLOR:
+        printf ( "rgba ( %.0f, %.0f, %.0f, %.0f %% )",
+                 ( p->value.color.red * 255.0 ),
+                 ( p->value.color.green * 255.0 ),
+                 ( p->value.color.blue * 255.0 ),
+                 ( p->value.color.alpha * 100.0 ) );
+        break;
+    case P_PADDING:
+        if ( distance_compare ( p->value.padding.top, p->value.padding.bottom ) &&
+             distance_compare ( p->value.padding.left, p->value.padding.right ) &&
+             distance_compare ( p->value.padding.left, p->value.padding.top ) ) {
+            rofi_theme_print_distance ( p->value.padding.left );
+        }
+        else if ( distance_compare ( p->value.padding.top, p->value.padding.bottom ) &&
+                  distance_compare ( p->value.padding.left, p->value.padding.right ) ) {
+            rofi_theme_print_distance ( p->value.padding.top );
+            rofi_theme_print_distance ( p->value.padding.left );
+        }
+        else if ( !distance_compare ( p->value.padding.top, p->value.padding.bottom ) &&
+                  distance_compare ( p->value.padding.left, p->value.padding.right ) ) {
+            rofi_theme_print_distance ( p->value.padding.top );
+            rofi_theme_print_distance ( p->value.padding.left );
+            rofi_theme_print_distance ( p->value.padding.bottom );
+        }
+        else {
+            rofi_theme_print_distance ( p->value.padding.top );
+            rofi_theme_print_distance ( p->value.padding.right );
+            rofi_theme_print_distance ( p->value.padding.bottom );
+            rofi_theme_print_distance ( p->value.padding.left );
+        }
+        break;
+    case P_LINK:
+        if ( p->value.link.def_value ) {
+            printf ( "var( %s, ", p->value.link.name );
+            int_rofi_theme_print_property ( p->value.link.def_value );
+            printf ( ")" );
+        }
+        else {
+            printf ( "var(%s)", p->value.link.name );
+        }
+        break;
+    case P_INHERIT:
+        printf ( "inherit" );
+        break;
+    default:
+        break;
     }
-
 }
 
 static void rofi_theme_print_property_index ( size_t pnl, int depth, Property *p )
@@ -462,7 +628,7 @@ static void rofi_theme_resolve_link_property ( Property *p, int depth )
 {
     // Set name, remove '@' prefix.
     const char *name = p->value.link.name;// + (*(p->value.link.name)== '@'?1:0;
-    g_info ( "Resolving link to %s", p->value.link.name);
+    g_info ( "Resolving link to %s", p->value.link.name );
     if ( depth > 20 ) {
         g_warning ( "Found more then 20 redirects for property. Stopping." );
         p->value.link.ref = p;
@@ -471,7 +637,7 @@ static void rofi_theme_resolve_link_property ( Property *p, int depth )
 
     if ( rofi_theme->properties && g_hash_table_contains ( rofi_theme->properties, name ) ) {
         Property *pr = g_hash_table_lookup ( rofi_theme->properties, name );
-        g_info ("Resolving link %s found: %s", p->value.link.name, pr->name);
+        g_info ( "Resolving link %s found: %s", p->value.link.name, pr->name );
         if ( pr->type == P_LINK ) {
             if ( pr->value.link.ref == NULL ) {
                 rofi_theme_resolve_link_property ( pr, depth + 1 );
@@ -487,7 +653,7 @@ static void rofi_theme_resolve_link_property ( Property *p, int depth )
         }
     }
     // No found and we have default value.
-    if ( p->value.link.def_value ){
+    if ( p->value.link.def_value ) {
         p->value.link.ref = p->value.link.def_value;
         return;
     }
@@ -585,17 +751,17 @@ RofiDistance rofi_theme_get_distance ( const widget *widget, const char *propert
             if ( widget->parent ) {
                 return rofi_theme_get_distance ( widget->parent, property, def );
             }
-            return (RofiDistance){ def, ROFI_PU_PX, ROFI_HL_SOLID };
+            return (RofiDistance){ .base = { def, ROFI_PU_PX, ROFI_DISTANCE_MODIFIER_NONE, NULL, NULL }, .style = ROFI_HL_SOLID };
         }
         if ( p->type == P_INTEGER ) {
-            return (RofiDistance){ p->value.i, ROFI_PU_PX, ROFI_HL_SOLID };
+            return (RofiDistance){ .base = { p->value.i, ROFI_PU_PX, ROFI_DISTANCE_MODIFIER_NONE, NULL, NULL }, .style = ROFI_HL_SOLID };
         }
         else {
             return p->value.padding.left;
         }
     }
     g_debug ( "Theme entry: #%s %s property %s unset.", widget->name, widget->state ? widget->state : "", property );
-    return (RofiDistance){ def, ROFI_PU_PX, ROFI_HL_SOLID };
+    return (RofiDistance){ .base = { def, ROFI_PU_PX, ROFI_DISTANCE_MODIFIER_NONE, NULL, NULL }, .style = ROFI_HL_SOLID };
 }
 
 int rofi_theme_get_boolean ( const widget *widget, const char *property, int def )
@@ -711,7 +877,7 @@ RofiPadding rofi_theme_get_padding ( const widget *widget, const char *property,
             pad = p->value.padding;
         }
         else {
-            RofiDistance d = (RofiDistance){ p->value.i, ROFI_PU_PX, ROFI_HL_SOLID };
+            RofiDistance d = (RofiDistance){ .base = { p->value.i, ROFI_PU_PX, ROFI_DISTANCE_MODIFIER_NONE, NULL, NULL }, .style = ROFI_HL_SOLID };
             return (RofiPadding){ d, d, d, d };
         }
     }
@@ -762,27 +928,74 @@ RofiHighlightColorStyle rofi_theme_get_highlight ( widget *widget, const char *p
     return th;
 }
 
-int distance_get_pixel ( RofiDistance d, RofiOrientation ori )
+static int get_pixels ( RofiDistanceUnit *unit, RofiOrientation ori )
 {
-    if ( d.type == ROFI_PU_EM ) {
-        return d.distance * textbox_get_estimated_char_height ();
+    int val = unit->distance;
+
+    if ( unit->type == ROFI_PU_EM ) {
+        val = unit->distance * textbox_get_estimated_char_height ();
     }
-    else if ( d.type == ROFI_PU_CH ) {
-        return d.distance * textbox_get_estimated_ch ();
+    else if ( unit->type == ROFI_PU_CH ) {
+        val = unit->distance * textbox_get_estimated_ch ();
     }
-    else if ( d.type == ROFI_PU_PERCENT ) {
+    else if ( unit->type == ROFI_PU_PERCENT ) {
         if ( ori == ROFI_ORIENTATION_VERTICAL ) {
             int height = 0;
             rofi_view_get_current_monitor ( NULL, &height );
-            return ( d.distance * height ) / ( 100.0 );
+            val = ( unit->distance * height ) / ( 100.0 );
         }
         else {
             int width = 0;
             rofi_view_get_current_monitor ( &width, NULL );
-            return ( d.distance * width ) / ( 100.0 );
+            val = ( unit->distance * width ) / ( 100.0 );
         }
     }
-    return d.distance;
+    else if ( unit->type == ROFI_PU_MM ) {
+        val = unit->distance * config.dpi / 25.4;
+    }
+    return val;
+}
+
+static int distance_unit_get_pixel ( RofiDistanceUnit *unit, RofiOrientation ori )
+{
+    switch ( unit->modtype )
+    {
+    case ROFI_DISTANCE_MODIFIER_GROUP:
+        return distance_unit_get_pixel ( unit->left, ori );
+        break;
+    case ROFI_DISTANCE_MODIFIER_ADD:
+        return distance_unit_get_pixel ( unit->left, ori ) + distance_unit_get_pixel ( unit->right, ori );
+    case ROFI_DISTANCE_MODIFIER_SUBTRACT:
+        return distance_unit_get_pixel ( unit->left, ori ) - distance_unit_get_pixel ( unit->right, ori );
+    case ROFI_DISTANCE_MODIFIER_MULTIPLY:
+        return distance_unit_get_pixel ( unit->left, ori ) * distance_unit_get_pixel ( unit->right, ori );
+    case ROFI_DISTANCE_MODIFIER_DIVIDE:
+    {
+        int a = distance_unit_get_pixel ( unit->left, ori );
+        int b = distance_unit_get_pixel ( unit->right, ori );
+        if ( b != 0 ) {
+            return a / b;
+        }
+        return a;
+    }
+    case ROFI_DISTANCE_MODIFIER_MODULO:
+    {
+        int a = distance_unit_get_pixel ( unit->left, ori );
+        int b = distance_unit_get_pixel ( unit->right, ori );
+        if ( b != 0 ) {
+            return a % b;
+        }
+        return 0;
+    }
+    default:
+        break;
+    }
+    return get_pixels ( unit, ori );
+}
+
+int distance_get_pixel ( RofiDistance d, RofiOrientation ori )
+{
+    return distance_unit_get_pixel ( &( d.base ), ori );
 }
 
 void distance_get_linestyle ( RofiDistance d, cairo_t *draw )
@@ -804,6 +1017,15 @@ gboolean rofi_theme_is_empty ( void )
     if ( rofi_theme->properties == NULL && rofi_theme->num_widgets == 0 ) {
         return TRUE;
     }
+    if ( rofi_theme->num_widgets == 3 ) {
+        // HACK: check for default added elements.
+        for ( unsigned int i = 0; i < rofi_theme->num_widgets; i++ ) {
+            if ( strncmp ( rofi_theme->widgets[i]->name, "element", 7 ) != 0 ) {
+                return FALSE;
+            }
+        }
+        return TRUE;
+    }
 
     return FALSE;
 }
@@ -965,3 +1187,152 @@ char * rofi_theme_parse_prepare_file ( const char *file, const char *parent_file
 
     return filename;
 }
+
+void rofi_theme_parse_merge_widgets ( ThemeWidget *parent, ThemeWidget *child )
+{
+    g_assert ( parent != NULL );
+    g_assert ( child != NULL );
+
+    if ( parent == rofi_theme && g_strcmp0 ( child->name, "*" ) == 0 ) {
+        rofi_theme_widget_add_properties ( parent, child->properties );
+        return;
+    }
+
+    ThemeWidget *w = rofi_theme_find_or_create_name ( parent, child->name );
+    rofi_theme_widget_add_properties ( w, child->properties );
+    for ( unsigned int i = 0; i < child->num_widgets; i++ ) {
+        rofi_theme_parse_merge_widgets ( w, child->widgets[i] );
+    }
+}
+
+void  rofi_theme_parse_process_conditionals ( void )
+{
+    workarea mon;
+    monitor_active ( &mon );
+    if ( rofi_theme == NULL ) {
+        return;
+    }
+    for ( unsigned int i = 0; i < rofi_theme->num_widgets; i++ ) {
+        ThemeWidget *widget = rofi_theme->widgets[i];
+        if ( widget->media != NULL ) {
+            switch ( widget->media->type )
+            {
+            case THEME_MEDIA_TYPE_MIN_WIDTH:
+            {
+                int w = widget->media->value;
+                if ( mon.w >= w ) {
+                    for ( unsigned int x = 0; x < widget->num_widgets; x++ ) {
+                        rofi_theme_parse_merge_widgets ( rofi_theme, widget->widgets[x] );
+                    }
+                }
+                break;
+            }
+            case THEME_MEDIA_TYPE_MAX_WIDTH:
+            {
+                int w = widget->media->value;
+                if ( mon.w < w ) {
+                    for ( unsigned int x = 0; x < widget->num_widgets; x++ ) {
+                        rofi_theme_parse_merge_widgets ( rofi_theme, widget->widgets[x] );
+                    }
+                }
+                break;
+            }
+            case THEME_MEDIA_TYPE_MIN_HEIGHT:
+            {
+                int h = widget->media->value;
+                if ( mon.h >= h ) {
+                    for ( unsigned int x = 0; x < widget->num_widgets; x++ ) {
+                        rofi_theme_parse_merge_widgets ( rofi_theme, widget->widgets[x] );
+                    }
+                }
+                break;
+            }
+            case THEME_MEDIA_TYPE_MAX_HEIGHT:
+            {
+                int h = widget->media->value;
+                if ( mon.h < h ) {
+                    for ( unsigned int x = 0; x < widget->num_widgets; x++ ) {
+                        rofi_theme_parse_merge_widgets ( rofi_theme, widget->widgets[x] );
+                    }
+                }
+                break;
+            }
+            case THEME_MEDIA_TYPE_MON_ID:
+            {
+                if ( mon.monitor_id == widget->media->value ) {
+                    for ( unsigned int x = 0; x < widget->num_widgets; x++ ) {
+                        rofi_theme_parse_merge_widgets ( rofi_theme, widget->widgets[x] );
+                    }
+                }
+                break;
+            }
+            case THEME_MEDIA_TYPE_MIN_ASPECT_RATIO:
+            {
+                double r = widget->media->value;
+                if ( ( mon.w / (double) mon.h ) >= r ) {
+                    for ( unsigned int x = 0; x < widget->num_widgets; x++ ) {
+                        rofi_theme_parse_merge_widgets ( rofi_theme, widget->widgets[x] );
+                    }
+                }
+                break;
+            }
+            case THEME_MEDIA_TYPE_MAX_ASPECT_RATIO:
+            {
+                double r = widget->media->value;
+                if ( ( mon.w / (double) mon.h ) < r ) {
+                    for ( unsigned int x = 0; x < widget->num_widgets; x++ ) {
+                        rofi_theme_parse_merge_widgets ( rofi_theme, widget->widgets[x] );
+                    }
+                }
+                break;
+            }
+            default:
+            {
+                break;
+            }
+            }
+        }
+    }
+}
+
+ThemeMediaType rofi_theme_parse_media_type ( const char *type )
+{
+    if ( g_strcmp0 ( type, "monitor-id" ) == 0 ) {
+        return THEME_MEDIA_TYPE_MON_ID;
+    }
+    else if ( g_strcmp0 ( type, "min-width" ) == 0 ) {
+        return THEME_MEDIA_TYPE_MIN_WIDTH;
+    }
+    else if ( g_strcmp0 ( type, "min-height" ) == 0 ) {
+        return THEME_MEDIA_TYPE_MIN_HEIGHT;
+    }
+    else if ( g_strcmp0 ( type, "max-width" ) == 0 ) {
+        return THEME_MEDIA_TYPE_MAX_WIDTH;
+    }
+    else if ( g_strcmp0 ( type, "max-height" ) == 0 ) {
+        return THEME_MEDIA_TYPE_MAX_HEIGHT;
+    }
+    else if ( g_strcmp0 ( type, "min-aspect-ratio" ) == 0 ) {
+        return THEME_MEDIA_TYPE_MIN_ASPECT_RATIO;
+    }
+    else if ( g_strcmp0 ( type, "max-aspect-ratio" ) == 0 ) {
+        return THEME_MEDIA_TYPE_MAX_ASPECT_RATIO;
+    }
+    return THEME_MEDIA_TYPE_INVALID;
+}
+
+gboolean rofi_theme_has_property ( const widget *widget, const char *property )
+{
+    ThemeWidget *wid = rofi_theme_find_widget ( widget->name, widget->state, FALSE );
+    Property    *p   = rofi_theme_find_property ( wid, P_STRING, property, FALSE );
+    if ( p ) {
+        if ( p->type == P_INHERIT ) {
+            if ( widget->parent ) {
+                return rofi_theme_has_property ( widget->parent, property );
+            }
+            return FALSE;
+        }
+        return TRUE;
+    }
+    return FALSE;
+}
