@@ -2,7 +2,7 @@
  * rofi
  *
  * MIT/X11 License
- * Copyright © 2013-2017 Qball Cow <qball@gmpclient.org>
+ * Copyright © 2013-2020 Qball Cow <qball@gmpclient.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the
@@ -107,10 +107,6 @@ static int box_get_desired_height ( widget *wid )
                 continue;
             }
             active_widgets++;
-            if ( child->expand == TRUE ) {
-                height += widget_get_desired_height ( child );
-                continue;
-            }
             height += widget_get_desired_height ( child );
         }
         if ( active_widgets > 0 ) {
@@ -229,12 +225,15 @@ static void hori_calculate_size ( box *b )
     if ( b->max_size > ( rem_width ) ) {
         b->max_size = rem_width;
         g_debug ( "Widgets to large (width) for box: %d %d", b->max_size, b->widget.w );
-        return;
+        //return;
     }
     if ( active_widgets > 0 ) {
         int    left  = widget_padding_get_left ( WIDGET ( b ) );
         double rem   = rem_width - b->max_size;
         int    index = 0;
+        if ( rem < 0 ) {
+            rem = 0;
+        }
         for ( GList *iter = g_list_first ( b->children ); iter != NULL; iter = g_list_next ( iter ) ) {
             widget * child = (widget *) iter->data;
             if ( child->enabled == FALSE  ) {
@@ -333,6 +332,14 @@ static widget *box_find_mouse_target ( widget *wid, WidgetType type, gint x, gin
     return NULL;
 }
 
+static void box_set_state ( widget *wid, const char *state )
+{
+    for ( GList *iter = g_list_first ( ( (box *) wid )->children ); iter != NULL; iter = g_list_next ( iter ) ) {
+        widget * child = (widget *) iter->data;
+        widget_set_state ( child, state );
+    }
+}
+
 box * box_create ( widget *parent, const char *name, RofiOrientation type )
 {
     box *b = g_malloc0 ( sizeof ( box ) );
@@ -346,6 +353,7 @@ box * box_create ( widget *parent, const char *name, RofiOrientation type )
     b->widget.find_mouse_target  = box_find_mouse_target;
     b->widget.get_desired_height = box_get_desired_height;
     b->widget.get_desired_width  = box_get_desired_width;
+    b->widget.set_state          = box_set_state;
 
     b->type = rofi_theme_get_orientation ( WIDGET ( b ), "orientation", b->type );
 
@@ -365,4 +373,7 @@ static void box_update ( widget *wid  )
     default:
         hori_calculate_size ( b );
     }
+    if ( wid->parent ) {
+        widget_update ( wid->parent );
+    }
 }
