@@ -3,7 +3,7 @@
  *
  * MIT/X11 License
  * Copyright © 2012 Sean Pringle <sean.pringle@gmail.com>
- * Copyright © 2013-2017 Qball Cow <qball@gmpclient.org>
+ * Copyright © 2013-2020 Qball Cow <qball@gmpclient.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the
@@ -99,6 +99,76 @@ static xcb_visualtype_t *root_visual = NULL;
 xcb_atom_t              netatoms[NUM_NETATOMS];
 const char              *netatom_names[] = { EWMH_ATOMS ( ATOM_CHAR ) };
 
+static xcb_visualtype_t * lookup_visual ( xcb_screen_t   *s, xcb_visualid_t visual )
+{
+    xcb_depth_iterator_t d;
+    d = xcb_screen_allowed_depths_iterator ( s );
+    for (; d.rem; xcb_depth_next ( &d ) ) {
+        xcb_visualtype_iterator_t v = xcb_depth_visuals_iterator ( d.data );
+        for (; v.rem; xcb_visualtype_next ( &v ) ) {
+            if ( v.data->visual_id == visual ) {
+                return v.data;
+            }
+        }
+    }
+    return 0;
+}
+
+cairo_surface_t *x11_helper_get_screenshot_surface_window ( xcb_window_t window, int size )
+{
+    xcb_get_geometry_cookie_t cookie;
+    xcb_get_geometry_reply_t  *reply;
+
+    cookie = xcb_get_geometry ( xcb->connection, window );
+    reply  = xcb_get_geometry_reply ( xcb->connection, cookie, NULL );
+    if ( reply == NULL ) {
+        return NULL;
+    }
+
+    xcb_get_window_attributes_cookie_t attributesCookie = xcb_get_window_attributes ( xcb->connection, window );
+    xcb_get_window_attributes_reply_t  *attributes      = xcb_get_window_attributes_reply ( xcb->connection,
+                                                                                            attributesCookie,
+                                                                                            NULL );
+    if ( attributes == NULL || ( attributes->map_state != XCB_MAP_STATE_VIEWABLE ) ) {
+        free ( reply );
+        if ( attributes ) {
+            free ( attributes );
+        }
+        return NULL;
+    }
+    // Create a cairo surface for the window.
+    xcb_visualtype_t * vt = lookup_visual ( xcb->screen, attributes->visual );
+    free ( attributes );
+
+    cairo_surface_t *t = cairo_xcb_surface_create ( xcb->connection, window, vt, reply->width, reply->height );
+
+    if ( cairo_surface_status ( t ) != CAIRO_STATUS_SUCCESS ) {
+        cairo_surface_destroy ( t );
+        free ( reply );
+        return NULL;
+    }
+
+    // Scale the image, as we don't want to keep large one around.
+    int             max   = MAX ( reply->width, reply->height );
+    double          scale = (double) size / max;
+
+    cairo_surface_t *s2 = cairo_surface_create_similar_image ( t, CAIRO_FORMAT_ARGB32, reply->width * scale, reply->height * scale );
+    free ( reply );
+
+    if ( cairo_surface_status ( s2 ) != CAIRO_STATUS_SUCCESS ) {
+        cairo_surface_destroy ( t );
+        return NULL;
+    }
+    // Paint it in.
+    cairo_t *d = cairo_create ( s2 );
+    cairo_scale ( d, scale, scale );
+    cairo_set_source_surface ( d, t, 0, 0 );
+    cairo_paint ( d );
+    cairo_destroy ( d );
+
+    cairo_surface_destroy ( t );
+    return s2;
+}
 /**
  * Holds for each supported modifier the possible modifier mask.
  * Check x11_mod_masks[MODIFIER]&mask != 0 to see if MODIFIER is activated.
@@ -988,8 +1058,8 @@ static int take_pointer ( xcb_window_t w, int iters )
             break;
         }
         struct timespec del = {
-             .tv_sec  = 0,
-             .tv_nsec =  1000000
+            .tv_sec  = 0,
+            .tv_nsec = 1000000
         };
         nanosleep ( &del, NULL );
     }
@@ -1019,8 +1089,8 @@ static int take_keyboard ( xcb_window_t w, int iters )
             break;
         }
         struct timespec del = {
-             .tv_sec  = 0,
-             .tv_nsec =  1000000
+            .tv_sec  = 0,
+            .tv_nsec = 1000000
         };
         nanosleep ( &del, NULL );
     }
