@@ -1,7 +1,6 @@
 #!/usr/bin/env bash
 
-xrdb -load ../doc/example.xresources 
-echo -e -n "aap\nnoot\nmies" | rofi -width -30 -dmenu > output.txt & 
+echo -e -n "aap\nnoot\nmies" | rofi -width -30 -dmenu -multi-select > output.txt & 
 RPID=$!
 
 # send enter.
@@ -18,15 +17,14 @@ xdotool key 'e'
 sleep 0.2
 xdotool key 'e'
 sleep 0.2
-xdotool key Shift+Return
 xdotool key Return
 
 
 #  Get result, kill xvfb
 wait ${RPID}
 RETV=$?
-OUTPUT=$(cat output.txt | tr '\n' ' ')
-if [ "${OUTPUT}" != 'coffee coffee ' ]
+OUTPUT=$( tr '\n' ' ' < output.txt )
+if [ "${OUTPUT}" != 'coffee ' ]
 then
     exit 1
 fi
