@@ -2,7 +2,7 @@
 
 # wait till it is up, run rofi with error message
 sleep 1;
-echo -e -n "aap\nnoot\nmies" | rofi -dmenu  > output.txt & 
+echo -e -n "aap\nnoot\nmies" | rofi -dmenu  -multi-select > output.txt & 
 RPID=$!
 
 # send enter.
@@ -10,12 +10,13 @@ sleep 5;
 xdotool key 'Down' 
 sleep 0.4
 xdotool key Shift+Return
+xdotool key Shift+Return
 xdotool key Return
 
 #  Get result, kill xvfb
 wait ${RPID}
 RETV=$?
-OUTPUT=$(cat output.txt | tr '\n' ' ')
+OUTPUT=$( tr '\n' ' ' < output.txt )
 if [ "${OUTPUT}" != 'noot mies ' ]
 then
     echo "Got: '${OUTPUT}' expected 'noot mies '"
