@@ -2,10 +2,10 @@
 
 SP=$(readlink -f "$0")
 DIR=$(dirname "$SP")
-echo $DIR
+echo "$DIR/test_script.sh"
 # wait till it is up, run rofi with error message
 sleep 1;
-rofi -switchers "custom:$DIR/test_script.sh" -show custom & 
+rofi -modi "custom:$DIR/test_script.sh" -show custom & 
 RPID=$!
 
 # send enter.
@@ -17,7 +17,8 @@ xdotool key Return
 #  Get result, kill xvfb
 wait ${RPID}
 RETV=$?
-OUTPUT=$(cat output.txt | tr '\n' ' ')
+OUTPUT=$( tr '\n' ' ' < output.txt )
+echo ${OUTPUT}
 if [ "${OUTPUT}" != 'noot ' ]
 then
     exit 1
